#include "../support/logger.h"
#include "free-tree.h"

MallocTable * table;

void InitializeMallocTable() {
    table = malloc(sizeof(MallocTable));
    table->elements = malloc(DEFAULT_TABLE_SIZE * sizeof(void *));
    table->size = DEFAULT_TABLE_SIZE;
    table->used = 0;
}

void AddMalloc(void * pointer) {
    if (table->used == table->size) {
        table->elements = realloc(table->elements, 2 * table->size);
        table->size *= 2;
    }
    table->elements[table->used++] = pointer;
}

void FreeMallocTable() {
    for (int i = 0; i < table->used; i++) {
        free(table->elements[i]);
    }
    free(table->elements);
    free(table);
}

// void FreeDerivative(Derivative * derivative) {
//     free(derivative->function);
//     free(derivative->grade);
//     FreeExpression(derivative->value);
//     free(derivative);
// }

// void FreeIntegrate(Integrate * integrate) {
//     free(integrate->function);
//     FreeExpression(integrate->lvalue);
//     FreeExpression(integrate->rvalue);
//     if (integrate->error != NULL)
//         free(integrate->error);
//     free(integrate);
// }

// void FreeEvaluate(Evaluate * evaluate) {
//     free(evaluate->function);
//     FreeExpression(evaluate->value);
//     free(evaluate);
// }

// void FreeExpression(Expression * expression) {
//     switch (expression->type) {
//         case DERIVATIVE_EXPRESSION:
//             FreeDerivative(expression->derivative);
//             break;
//         case INTEGRATE_EXPRESSION:
//             FreeIntegrate(expression->integrate);
//             break;
//         case EVALUATE_EXPRESSION:
//             FreeEvaluate(expression->evaluate);
//             break;
//         case NEGATIVE_EXPRESSION:
//         case PARENTHESIS_EXPRESSION:
//             FreeExpression(expression->leftExpression);
//             break;
//         case NUMBER_EXPRESSION:
//             break;
//         case ADD_EXPRESSION:
//         case SUB_EXPRESSION:
//         case MUL_EXPRESSION:
//         case DIV_EXPRESSION:
//         case POW_EXPRESSION:
//             FreeExpression(expression->leftExpression);
//             FreeExpression(expression->rightExpression);
//             break;
//         case ID_EXPRESSION:
//             free(expression->id);
//             break;
//         default:
//             abort();
//     }
//     free(expression);
// }

// void FreeDomain(Domain * domain) {
//     switch (domain->type) {
//         case GREATER_DOMAIN:
//         case GREATER_EQUAL_DOMAIN:
//         case LESSER_EQUAL_DOMAIN:
//         case LESSER_DOMAIN:
//         case EQUAL_DOMAIN:
//         case NOT_EQUAL_DOMAIN:
//             FreeExpression(domain->left);
//             FreeExpression(domain->right);
//             break;
//         case LESSER_LESSER_DOMAIN:
//         case LESSER_LESSER_EQUAL_DOMAIN:
//         case LESSER_EQUAL_LESSER_EQUAL_DOMAIN:
//         case LESSER_EQUAL_LESSER_DOMAIN:
// //            FreeExpression(domain->left);
// //            FreeExpression(domain->mid);
// //            FreeExpression(domain->right);
//             break;
//         default:
//             abort();
//             break;
//     }
//     free(domain);
// }

// void FreeDomainForFunction(Domain * domain) {
//     switch (domain->type) {
//         case GREATER_DOMAIN:
//         case GREATER_EQUAL_DOMAIN:
//         case LESSER_EQUAL_DOMAIN:
//         case LESSER_DOMAIN:
//         case EQUAL_DOMAIN:
//         case NOT_EQUAL_DOMAIN:
//             FreeExpression(domain->left);
//             FreeExpression(domain->right);
//             break;
//         case LESSER_LESSER_DOMAIN:
//         case LESSER_LESSER_EQUAL_DOMAIN:
//         case LESSER_EQUAL_LESSER_EQUAL_DOMAIN:
//         case LESSER_EQUAL_LESSER_DOMAIN:
//             FreeExpression(domain->left);
//             FreeExpression(domain->mid);
//             FreeExpression(domain->right);
//             break;
//     }
//     free(domain);
// }

// void FreeBoolean(Boolean * boolean) {
//     switch (boolean->type) {
//     case DOMAIN_BOOLEAN:
//         FreeDomain(boolean->domain);
//         break;
//     case OR_BOOLEAN:
//     case AND_BOOLEAN:
//         FreeBoolean(boolean->left);
//         FreeBoolean(boolean->right);
//         break;
//     case NOT_BOOLEAN:
//     case BOOLEAN:
//         FreeBoolean(boolean->left);
//         break;
//     default:
//         abort();
//     }
//     free(boolean);
// }

// void FreeIf(If * if_st) {
//     FreeBoolean(if_st->boolean);
//     FreeBlock(if_st->block);
//     if (if_st->else_block != NULL) {
//         FreeBlock(if_st->else_block);
//     }
//     free(if_st);
// }

// void FreeFor(For * for_st) {
//     if (for_st->init_variable != NULL)
//         FreeDeclareVariable(for_st->init_variable);
//     FreeBoolean(for_st->boolean);
//     FreeAssignVariable(for_st->action_variable);
//     FreeBlock(for_st->block);
//     free(for_st);
// }


// void FreeBlock(Block * block) {
//     Statement * statement = block->statement, * aux;
//     while (statement != NULL) {
//         aux = statement;
//         statement = statement->next;
//         FreeStatement(aux);
//     }
//     free(block);
// }

// void FreeDeclareVariable(Variable * variable) {
//     free(variable->name);
//     if (variable->value != NULL) {
//         FreeExpression(variable->value);
//     }
//     free(variable);
// }

// void FreeAssignVariable(AssignVariable * variable) {
//     switch (variable->type) {
//         case INCREMENT_EXPRESSION:
//         case DECREMENT_EXPRESSION:
//             break;
//         case ADD_ASSIGN_EXPRESSION:
//         case SUB_ASSIGN_EXPRESSION:
//         case MUL_ASSIGN_EXPRESSION:
//         case DIV_ASSIGN_EXPRESSION:
//         case ASSIGN_EXPRESSION:
//             FreeExpression(variable->expression);
//             break;
//         default:
//             abort();
//     }
//     free(variable->id);
//     free(variable);
// }

// void FreePrint(Print * print_st) {
//     switch (print_st->type) {
//         case STRING_PRINT:
//             free(print_st->string);
//             break;
//         case EXPRESSION_PRINT:
//             FreeExpression(print_st->expression);
//             break;
//         default:
//             abort();
//     }
//     free(print_st);
// }

// void FreeWhile(While * while_st) {
//     FreeBoolean(while_st->boolean);
//     FreeBlock(while_st->block);
//     free(while_st);
// }

// void FreeCompositeFunction(Composite * composite) {
//     switch (composite->type) {
//         case ID_COMPOSITE:
//             free(composite->inner_func);;
//             break;
//         case COMPOSITE_COMPOSITE:
//             FreeCompositeFunction(composite->inner_comp);
//             break;
//         default:
//             LogError("Invalid composite function");
//             abort();
//     }
//     free(composite->outter_func);
//     free(composite);
// }

// void FreeDeclareFunction(Function * function) {
//     Function * currFunc = function;
//     free(function->name);
//     Function * aux;
//     while (currFunc != NULL) {
//         switch (currFunc->type) {
//             case EXPRESSION_FUNCTION:
//                 FreeExpression(currFunc->expression);
//                 break;
//             case EXPRESSION_DOMAIN_FUNCTION:
//                 FreeExpression(currFunc->expression);
//                 FreeDomainForFunction(currFunc->domain);
//                 break;
//             case COMPOSITE_FUNCTION:
//                 FreeCompositeFunction(currFunc->composite);
//                 break;
//             default:
//                 LogError("Incorrect function definition");
//                 abort();
//         }
//         aux = currFunc;
//         currFunc = currFunc->next;
//         free(aux);
//     }
// }

// void FreeStatement(Statement * statement) {
//     switch (statement->type) {
//         case WHILE_STATEMENT:
//             FreeWhile(statement->while_st);
//             break;
//         case FOR_STATEMENT:
//             FreeFor(statement->for_st);
//             break;
//         case IF_STATEMENT:
//             FreeIf(statement->if_st);
//             break;
//         case PRINT_STATEMENT:
//             FreePrint(statement->print);
//             break;
//         case INTEGRATE_STATEMENT:
//             FreeIntegrate(statement->integrate);
//             break;
//         case DERIVATIVE_STATEMENT:
//             FreeDerivative(statement->derivative);
//             break;
//         case EVALUATE_STATEMENT:
//             FreeEvaluate(statement->evaluate);
//             break;
//         case DECLARE_VARIABLE_STATEMENT:
//             FreeDeclareVariable(statement->declareVariable);
//             break;
//         case ASSIGN_VARIABLE_STATEMENT:
//             FreeAssignVariable(statement->assignVariable);
//             break;
//         case DECLARE_FUNCTION_STATEMENT:
//             FreeDeclareFunction(statement->declareFunction);
//             break;
//         default:
//             LogError("Invalid statement");
//             abort();
//     }
//     free(statement);
// }

// void FreeTree(Program * program) {
//     if (program == NULL)
//         return;
//     Statement * currStatement = program->statement, * aux;
//     while (currStatement != NULL) {
//         aux = currStatement;
//         currStatement = currStatement->next;
//         FreeStatement(aux);
//     }
// }
